function [TP,R,nextLoc,Rjump,Rcont] = reach(obj,tStart,R0,options)
% reach - computes the reachable set of the system within a location, 
% detects the guard set that is hit and computes the reset
%
% Syntax:  
%    [TP,R,nextLoc,Rjump,Rcont] = reach(obj,tStart,R0,options)
%
% Inputs:
%    obj - location object
%    tStart - start time
%    R0 - initial reachable set
%    options - options struct
%
% Outputs:
%    TP - time point struct; e.g. contains time vector of minimum times for reaching guard sets
%    R - cell array of reachable sets
%    nextLoc - next location
%    Rjump - reachable set after jump according to the reset map
%    Rcont - reachable set due to continuous evolution
%
% Example: 
%
% Other m-files required: initReach, reach, potOut, potInt, guardIntersect,
% reset
% Subfunctions: none
% MAT-files required: none
%
% See also: none

% Author:       Matthias Althoff
% Written:      07-May-2007 
% Last update:  17-August-2007
%               26-March-2008
%               21-April-2009
%               06-July-2009
%               24-July-2009
%               31-July-2009
%               11-August-2010
%               22-October-2010
%               25-October-2010
%               08-August-2011
% Last revision: ---

%------------- BEGIN CODE --------------


%set up TPtmp
TP.tMin=[];
TP.tMax=[];
%set up Rjump, nextLoc
R{1}=[];
nextLoc=[];
Rjump=[];
Rcont{1}=[];

%obtain enclosing zonotopes
if options.isHybrid==0
    % get results from reachability analysis
    [TP,R,activeGuards,Rguard,Rcont] = singleSetReach(obj,tStart,R0,options);
else
    
    if options.isHyperplaneMap
        % get results from hyperplane mapping
        [TP,R,activeGuards,Rguard,Rcont] = hyperplaneMap(obj,tStart,R0,options);
    
    else
    
        %get results from reachability analysis
        [TP,R,activeGuards,Rguard,Rcont] = singleSetReach(obj,tStart,R0,options);

%         %update W matrix
%         A=get(obj.contDynamics,'A');
%         if isnumeric(A)
%             options.W{2}=expm(A*TP.tMin(1))+0.001*eye(length(A));
%         end

        %compute enclosing zonotopes
        for i=1:length(activeGuards)
            tic

            %obtain guard number
            iGuard = activeGuards(i);

            Rguard{iGuard} = enclosePolytopes(obj,Rguard{iGuard},options);
            %Rguard{iGuard} = enclosePolytopes_new(obj,Rguard{iGuard},options);

            disp('CLASSICAL GUARD INTERSECT TIME: ENCLOSURE')
            toc
        end
    end
    
    %compute jumps, next locations
    for i=1:length(activeGuards)
        
        %obtain guard number
        iGuard = activeGuards(i);
        
        %compute reset and new locations of active guards
        Rjump{i}=reset(obj.transition{iGuard},Rguard{iGuard});  
        nextLoc(i)=get(obj.transition{iGuard},'target');
    end
end



%------------- END OF CODE --------------