function [Z_error, errorInt, errorInt_y, R_y] = linError(obj, options, R, Verror_y)
% linError - computes the linearization error
%
% Syntax:  
%    [obj] = linError(obj,options)
%
% Inputs:
%    obj - nonlinear system object
%    options - options struct
%    R - actual reachable set
%
% Outputs:
%    obj - nonlinear system object
%
% Example: 
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% See also: 

% Author:       Matthias Althoff
% Written:      29-October-2007 
% Last update:  22-January-2008
%               02-February-2010
% Last revision: ---

%------------- BEGIN CODE --------------

%compute set of algebraic variables
f0_con = obj.linError.f0_con;
D = obj.linError.D;
E = obj.linError.E;
F_inv = obj.linError.F_inv;
R_y = -F_inv*(f0_con + D*R + E*options.U + Verror_y);

%compute intervalhull of reachable set
IH_x = intervalhull(R);
totalInt_x = interval(IH_x) + obj.linError.p.x;

%compute intervalhull of algebraic states
IH_y = intervalhull(R_y);
totalInt_y = interval(IH_y) + obj.linError.p.y;

%compute intervals of input
IH_u = intervalhull(options.U);
totalInt_u = interval(IH_u) + obj.linError.p.u;

% %obtain maximum absolute values within IH_x
% IH_x_inf = abs(inf(IH_x));
% IH_x_sup = abs(sup(IH_x));
% dx = max(IH_x_inf,IH_x_sup);
% 
% %obtain maximum absolute values within IH_y
% IH_y_inf = abs(inf(IH_y));
% IH_y_sup = abs(sup(IH_y));
% dy = max(IH_y_inf,IH_y_sup);
% 
% %obtain maximum absolute values within IH_y
% IH_u_inf = abs(inf(IH_u));
% IH_u_sup = abs(sup(IH_u));
% du = max(IH_u_inf,IH_u_sup);

%obtain intervals and combined interval z
dx = interval(IH_x);
dy = interval(IH_y);
du = interval(IH_u);
dz = [dx; dy; du];

tic
%obtain hessian tensor
[Hf, Hg] = hessianTensor(totalInt_x, totalInt_y, totalInt_u);
[Hf2, Hg2] = hessianTensor_optimized(totalInt_x, totalInt_y, totalInt_u);

%error_x
for i=1:length(Hf)
    error_x(i,1) = 0.5*dz'*Hf{i}*dz;
end

%error_y
for i=1:length(Hg)
    error_y(i,1) = 0.5*dz'*Hg{i}*dz;
end
toc

% %obtain intervals and combined interval z
% dx = sup(abs(interval(IH_x)));
% dy = sup(abs(interval(IH_y)));
% du = sup(abs(interval(IH_u)));
% dz = [dx; dy; du];
% 
% tic
% %obtain hessian tensor
% [Hf, Hg] = hessianTensor(totalInt_x, totalInt_y, totalInt_u);
% 
% %error_x
% for i=1:length(Hf)
%     error_x(i) = 0.5*dz'*sup(abs(Hf{i}))*dz;
% end
% 
% %error_y
% for i=1:length(Hg)
%     error_y(i) = 0.5*dz'*sup(abs(Hg{i}))*dz;
% end
% toc
% 
% %compute linearization error by passing the intervals to the Lagrange
% %remainder mFile
% tic
% [error_x, error_y] = lagrangeRemainder(totalInt_x,totalInt_y,totalInt_u,dx,dy,du);
% toc

%check if Verror_y was overapproximative
if ~(intervalhull([inf(error_y),sup(error_y)]) <= intervalhull(Verror_y))
    %constraint variable assumption violated
    error_x = inf*error_x;
    errorInt_x = inf*errorInt_x;
end

%compute final error
Z_error_x = zonotope(intervalhull([inf(error_x),sup(error_x)]));
Z_error_y = zonotope(intervalhull([inf(error_y),sup(error_y)]));
Z_error = Z_error_x + obj.linError.CF_inv*Z_error_y;

%update R_y
R_y =  obj.linError.p.y + (-F_inv)*(f0_con + D*R + E*options.U + Z_error_y);

%error intervals
errorIHabs = abs(intervalhull(Z_error));
errorInt = errorIHabs(:,2);

errorIHabs_y = abs(intervalhull(Z_error_y));
errorInt_y = errorIHabs_y(:,2);

%------------- END OF CODE --------------