function [error, errorInt, errorInt_y, R_y] = linError_mixed(obj, options, R, Verror_y)
% linError - computes the linearization error
%
% Syntax:  
%    [obj] = linError(obj,options)
%
% Inputs:
%    obj - nonlinear DAE system object
%    options - options struct
%    R - actual reachable set
%
% Outputs:
%    error - zonotope overapproximating the linearization error
%    errorInt - interval overapproximating the linearization error
%
% Example: 
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% See also: 

% Author:       Matthias Althoff
% Written:      21-November-2011
% Last update:  ---
% Last revision:---

%------------- BEGIN CODE --------------

%compute set of algebraic variables
f0_con = obj.linError.f0_con;
D = obj.linError.D;
E = obj.linError.E;
F_inv = obj.linError.F_inv;
R_y_cor = -F_inv*(f0_con + D*R); %correlated part
R_y_add = -F_inv*(E*options.U + Verror_y); %uncorrelated part

%compute intervalhull of reachable set
IH_x = intervalhull(R);
totalInt_x = interval(IH_x) + obj.linError.p.x;

%compute intervalhull of algebraic states
IH_y = intervalhull(R_y_cor + R_y_add);
totalInt_y = interval(IH_y) + obj.linError.p.y;

%compute intervals of input
IH_u = intervalhull(options.U);
totalInt_u = interval(IH_u) + obj.linError.p.u;

%obtain hessian tensor
%[Hf_old, Hg_old] = hessianTensor(totalInt_x, totalInt_y, totalInt_u);
[Hf, Hg] = hessianTensor_optimized(totalInt_x, totalInt_y, totalInt_u);


% correct = 1;
% step = 1;
% for step = 1 : length(Hf)
%     correct = isempty(find(Hf_old{step} ~= Hf{step}, 1));
%     if ~correct
%         disp(['check ',num2str(step),'; ',num2str(find(Hf_old{step} ~= Hf{step}, 1))]);
%     end
%     step = step + 1;
% end
% 
% Hg_old{11}(11,11) = 11;
% 
% correct = 1;
% step = 1;
% for step = 1 : length(Hg)
%     correct = isempty(find(Hg_old{step} ~= Hg{step}, 1));
%     if ~correct
%         disp(['check ',num2str(step),'; ',num2str(find(Hg_old{step} ~= Hg{step}, 1))]);
%     end
%     step = step + 1;
% end

%compute zonotope of state, constarint variables, and input
Z_x = get(R,'Z');
Z_y_cor = get(R_y_cor,'Z');
Z_y_add = get(R_y_add,'Z');
Z_0 = zeros(length(Z_x(:,1)), length(Z_y_add(1,:)));
R_xy = zonotope([Z_x, Z_0; Z_y_cor, Z_y_add]);
R_xyu = cartesianProduct(R_xy, options.U);
R_xyu = reduce(R_xyu,'girard',options.errorOrder);

%obtain intervals and combined interval z
dx = interval(IH_x);
dy = interval(IH_y);
du = interval(IH_u);
dz = [dx; dy; du];

% %compute input due to second order evaluation
% error_x = 0.5*quadraticMultiplication_interval(Z_xyu, Hf);
% error_y = 0.5*quadraticMultiplication_interval(Z_xyu, Hg);

%separate evaluation
for i=1:length(Hf)
    Hf_mid{i} = sparse(mid(Hf{i}));
    Hf_rad{i} = sparse(rad(Hf{i}));
end
for i=1:length(Hg)
    Hg_mid{i} = sparse(mid(Hg{i}));
    Hg_rad{i} = sparse(rad(Hg{i}));
end
%zonotope evaluation
error_x_mid = 0.5*quadraticMultiplication(R_xyu, Hf_mid);
error_y_mid = 0.5*quadraticMultiplication(R_xyu, Hg_mid);

%interval evaluation
for i=1:length(Hf)
    error_x_rad(i,1) = 0.5*dz'*Hf_rad{i}*dz;
end
for i=1:length(Hg)
    error_y_rad(i,1) = 0.5*dz'*Hg_rad{i}*dz;
end

%combine results
error_x_rad_zono = zonotope(intervalhull([inf(error_x_rad), sup(error_x_rad)]));
error_y_rad_zono = zonotope(intervalhull([inf(error_y_rad), sup(error_y_rad)]));
% error_x = error_x_mid + error_x_rad_zono;
error_y = error_y_mid + error_y_rad_zono;

%check if Verror_y was overapproximative
if ~(intervalhull(error_y) <= intervalhull(Verror_y))
    %constraint variable assumption violated
    error_x = inf*error_x;
    errorInt_x = inf*errorInt_x;
end

%compute final error: to be CHECKED IF CORRELATION APPLIES
Z_err_x_mid = get(error_x_mid,'Z');
Z_err_x_add_mid = get(obj.linError.CF_inv*error_y_mid,'Z');
error_mid = zonotope(Z_err_x_mid + Z_err_x_add_mid);
error_rad = error_x_rad_zono + obj.linError.CF_inv*error_y_rad_zono;
error = error_mid + error_rad;
%error = error_x + obj.linError.CF_inv*error_y;

%reduce
error = reduce(error,'girard',options.zonotopeOrder);
error_y = reduce(error_y,'girard',options.zonotopeOrder);

%update R_y
R_y =  obj.linError.p.y + (-F_inv)*(f0_con + D*R + E*options.U + error_y);

%error intervals
errorIHabs = abs(intervalhull(error));
errorInt = errorIHabs(:,2);

errorIHabs_y = abs(intervalhull(error_y));
errorInt_y = errorIHabs_y(:,2);


% H2 = hessianTensor(mid(totalInt),mid(inputInt));
% error2=0.5*quadraticMultiplication(Z,H2);

%------------- END OF CODE --------------