function [Zsplit] = split(Z,options,varargin)
% split - Splits a zonotope into two parallelpipeds that enclose the
% zonotope. This is done for every generator of the parallelpiped enclosing
% the original zonotope resulting in n possible splits where n is the
% system dimension
%
% Syntax:  
%    [Zsplit] = split(Z)
%
% Inputs:
%    Z - zonotope object
%
% Outputs:
%    Zsplit - cell array of parallelpipeds represented as zonotopes
%
% Example: 
%    Z=zonotope(rand(2,4));
%    Zsplit=split(Z);
%    plot(Z);
%    hold on
%    plot(Zsplit{1}{1});
%    plot(Zsplit{1}{2});
%    plot(Zsplit{2}{1});
%    plot(Zsplit{2}{2});
%
% Other m-files required: reduce
% Subfunctions: none
% MAT-files required: none
%
% See also: none

% Author:       Matthias Althoff
% Written:      04-January-2008
% Last update:  09-October-2008
%               26-June-2009
%               02-February-2011
% Last revision: ---

%------------- BEGIN CODE --------------

%initialize
maxOrder=1;
all=1;
if nargin==3
    dim=varargin{1};
    maxOrder=1;
    all=0;
elseif nargin==4
    dim=varargin{1};
    maxOrder=varargin{2};
    all=0;    
end

%overapproximate zonotope by parallelpiped (pp)
order=length(Z.Z(1,2:end))/length(Z.Z(:,1));

if order>maxOrder
    if maxOrder==1
        Zpp=zonotope(intervalhull(Z));
%         Zpp=pinv(options.W)*reduce(options.W*Z,'methC',1,options.filterLength);
%         Zpp=Z.O*intervalhull(pinv(Z.O)*Z);
%         ZmedRed = reduce(options.W*Z,'girard',2);
%         Zpp=pinv(options.W)*zonotope(vertices(ZmedRed));
    else
        Zpp=pinv(options.W)*reduce(options.W*Z,'methD',maxOrder);
        %Zpp=reduceGirard(Z,maxOrder); %<-- changed
    end
else
    Zpp=Z;
end

%obtain dimension, center and generator matrix of parallelpiped
c=Zpp.Z(:,1);
G=Zpp.Z(:,2:end);

if all
    for dim=1:length(G(1,:))
        %split one dimension
        Zsplit{dim}=splitOneDim(c,G,dim); 
    end
else
    %split one dimension
    Zsplit=splitOneDim(c,G,dim);  
end


function Zsplit=splitOneDim(c,G,dim)

%compute centers of splitted parallelpiped
c1=c-G(:,dim)/2;
c2=c+G(:,dim)/2;

%compute new set of generators
Gnew=G;
Gnew(:,dim)=Gnew(:,dim)/2;

%generate splitted parallelpipeds
Zsplit{1}=zonotope([c1,Gnew]);
Zsplit{2}=zonotope([c2,Gnew]);   


%------------- END OF CODE --------------